import java.util.Iterator;

/**
 * This generic class provides methods to manage, add and delete a list of lists of elements.
 * 
 * @author Marvin Hanke
 * 
 * @param <A>
 *            Type parameter for the elements contained in the list.
 * @param <B>
 *            Type parameter for the elements contained in the list of lists.
 */
public class AList<A,B> extends SList<A> {

	protected Node head = null; // first node of list (prev == null)
	protected Node tail = null; // last node of list (next == null)
	
	/**
	 * This private inner class provides nodes for the list of AList.
	 * 
	 * @author Marvin Hanke
	 * 
	 */
	protected class Node {
		protected A elem; // element in this node
		private SList<B> list; // list of associated elements in this node
		private Node prev = null; // previous node in list
		protected Node next = null; // next node in list

		/**
		 * This constructor creates a new node with the assigned element.
		 * 
		 * @param elem
		 *            Element which is about to be assigned to the new node.
		 */
		private Node(A elem) {
			this.elem = elem;
			this.list = null;
		}
		
		/**
		 * This constructor creates a new node with the assigned element.
		 * 
		 * @param elem
		 *            Element which is about to be assigned to the new node.
		 * @param assoElements
		 *            Elements which are about to be associated to the element in the new node.
		 */
		private Node(A elem, B[] assoElements) {
			this.elem = elem;
			this.list = new SList<B>();
			for (int i = 0; i < assoElements.length; i++) {
				list.add(assoElements[i], -1);
			}
		}
	}

	/**
	 * This method inserts a new element at the end of the list.
	 * 
	 * @param elem
	 *            Element which is about to be inserted to the list.
	 */
	public void add(A elem, B[] content) {
		add(elem, -1, content);
	}
	
	/**
	 * This method inserts a new element (list) at the specified position.
	 * 
	 * @param elem
	 *            Element (list) which is about to be inserted to the list.
	 * @param x
	 *            Specifies the position where the new element is about to be inserted:
 *                x == -1: insert is performed at the end of the list.
 *                x == 0: insert is performed at the beginning of the list.
 *                x > 0 (x must not exceed the number of elements in list!): insert is performed
 *                  directly after the node at position x (first node starts with value 1 (not 0)).
 *                x < -1 or x exceeds the number of elements in list: Exception!
	 * @param content
	 *            Elements contained in the element (list) which is about to be added.
	 */
	public void add(A elem, int x, B[] content) {

		if (size == 0 && (x == -1 || x == 0)) {
			head = new Node(elem, content);
			tail = head;
			size++;
			return;
		} else if (size == 0) {
			throw new IndexOutOfBoundsException();
		}

		if (x == -1 || x == size) {
			tail.next = new Node(elem, content);
			tail.next.prev = tail;
			tail = tail.next;
		} else if (x == 0) {
			head.prev = new Node(elem, content);
			head.prev.next = head;
			head = head.prev;
		} else if (x > 0 && x < size) {
			
			int counter = 0;
			Node temp = head; // temporary node

			// After this loop, temp points at the node after which the new element is about to be inserted.
			while (temp.next != null && counter < x - 1) {
				temp = temp.next;
				counter++;
			}

			Node puffer = temp.next;
			temp.next = new Node(elem, content);
			temp.next.prev = temp;
			temp.next.next = puffer;
			puffer.prev = temp.next;
		} else {
			throw new IndexOutOfBoundsException();
		}

		size++;
	}
	
	/**
	 * This private inner class provides an iterator for the list of associated elements of AList.
	 * 
	 * @author Marvin Hanke
	 *
	 */
	private class ListIter implements Iterable_Iterator<A,B> {
		private Node current_elem = head; // current position of iterator
		private int counter = 0; // counts the position of last

		/**
		 * This method returns the next element in the iteration.
		 * 
		 * @return next element in the iteration.
		 */
		public A next() {
			if (current_elem == null) {
				return null;
			}
			
			Node last = current_elem;
			current_elem = current_elem.next;
			counter++;
			return last.elem; 
		}

		/**
		 * This method checks if the iteration has more elements.
		 * 
		 * @return true if the iteration has more elements.
		 */
		public boolean hasNext() {
			return current_elem != null;
		}

		/**
		 * This method removes from the underlying collection the last element returned by the iterator.
		 */
		public void remove() {
			AList.this.remove(counter);
			counter--;
		}
		
		/**
		 * This method returns another iterator of the list of associated elements in the current element of AList.
		 * 
		 * @return iterator of associated elements.
		 */
		public Iterator<B> iterator() {
			if (current_elem == null) {
				return null;
			}
			
			return current_elem.list.iterator();
		}
	}
	
	/**
	 * This method creates an iterator for the list.
	 * 
	 * @return an iterator.
	 */
	public Iterable_Iterator<A,B> iterator() {
		return new ListIter();
	}
}
