/**
 * This abstract class provides methods to check dependency and track those checks for specific categories.
 * 
 * @author Boris Serdar
 * 
 */
public abstract class Category implements Dependent<Category> {

	private int usage = 0;

	/**
	 * This method checks dependency of the current object (this) with another one.
	 * 
	 * @param elem
	 *            Element for which dependency with "this" is checked.
	 * @return true if dependency exists, false otherwise. False is also returned, if the classes of both objects are
	 *         not equal.
	 */
	public final boolean dependsOn(Category elem) {
		if (elem != null && (this.getClass() != elem.getClass())) {
			return false;
		}

		this.incUsage();
		elem.incUsage();

		// pre-condition of dependsOnCalc is met! this and elem have the same class!
		return dependsOnCalc(elem);
	}

	/**
	 * This method calculates dependency of the current object (this) with another one. It supports the dependsOn
	 * method.
	 * Pre-condition of this method is, that the elem variable is instance of the same class as the current
	 * object (this)!
	 * 
	 * @param elem
	 *            Element for which dependency with "this" is checked.
	 * @return true if dependency exists, false otherwise.
	 */
	protected abstract boolean dependsOnCalc(Category elem);

	/**
	 * This method tracks the number of usages of the dependsOn method for this object.
	 * 
	 * @return number of usages of the dependsOn method for this object.
	 */
	public final int uses() {
		return usage;
	}

	/**
	 * This method increases the number of usages of the dependsOn method for this object by 1.
	 */
	public final void incUsage() {
		usage++;
	}

	/**
	 * This method creates a string representation of the current object (this).
	 * 
	 * @return representative string.
	 */
	public abstract String toString();
}
