//import java.lang.Iterable;
import java.util.Iterator;

/**
 * This generic class provides methods to manage, add and delete a list of elements.
 * 
 * @author Stefan Klbl
 * 
 * @param <A>
 *            Type parameter for the elements contained in the list.
 */
public class SList<A> implements java.lang.Iterable<A> {

	private Node head = null; // first node of list (prev == null)
	private Node tail = null; // last node of list (next == null)
	protected int size = 0; // size of list

	/**
	 * This private inner class provides nodes for the list of SList.
	 * 
	 * @author Stefan Klbl
	 * 
	 */
	private class Node {
		private A elem; // element in this node
		private Node prev = null; // previous node in list
		private Node next = null; // next node in list

		/**
		 * This constructor creates a new node with the assigned element.
		 * 
		 * @param elem
		 *            Element which is about to be assigned to the new node.
		 */
		private Node(A elem) {
			this.elem = elem;
		}
	}

	/**
	 * This method inserts a new element at the end of the list.
	 * 
	 * @param elem
	 *            Element which is about to be inserted to the list.
	 */
	public void add(A elem) {
		add(elem, -1);
	}

	/**
	 * This method inserts a new element at the specified position.
	 * 
	 * @param elem
	 *            Element which is about to be inserted to the list.
	 * @param x
	 *            Specifies the position where the new element is about to be inserted:
	 *            x == -1: insert is performed at the end of the list.
	 *            x == 0: insert is performed at the beginning of the list.
	 *            x > 0 (x must not exceed the number of elements in list!): insert is performed
	 *            	directly after the node at position x (first node starts with value 1 (not 0)).
	 *            x < -1 or x exceeds the number of elements in list: Exception!
	 */
	public void add(A elem, int x) {

		if (size == 0 && (x == -1 || x == 0)) {
			head = new Node(elem);
			tail = head;
			size++;
			return;
		} else if (size == 0) {
			throw new IndexOutOfBoundsException();
		}

		if (x == -1 || x == size) {
			tail.next = new Node(elem);
			tail.next.prev = tail;
			tail = tail.next;
		} else if (x == 0) {
			head.prev = new Node(elem);
			head.prev.next = head;
			head = head.prev;
		} else if (x > 0 && x < size) {
			
			int counter = 0;
			Node temp = head; // temporary node

			// After this loop, temp points at the node after which the new element is about to be inserted.
			while (temp.next != null && counter < x - 1) {
				temp = temp.next;
				counter++;
			}

			Node puffer = temp.next;
			temp.next = new Node(elem);
			temp.next.prev = temp;
			temp.next.next = puffer;
			puffer.prev = temp.next;
		} else {
			throw new IndexOutOfBoundsException();
		}

		size++;
	}
	
	/**
	 * This method calculates the size of the list.
	 * 
	 * @return size of list.
	 */
	public int getSize() {
		return size;
	}
	
	/**
	 * This method removes the element at the specified position. List must not be empty! The value of parameter x must
	 * be handled correctly, therfore see parameter x for detailled description.
	 * 
	 * @param x
	 *            Specifies the position where the element is about to be deleted:
	 *            x == -1: delete is performed at the last element.
	 *            x == 0: delete is performed at the first element.
	 *            x > 0 (x must not exceed the number of elements in list!): delete is performed at node at position x
	 *            (first node starts with value 1 (not 0)).
	 *            x < -1 or x exceeds the number of elements in list: Exception!
	 */
	public void remove(int x) {

		if (size == 0) {
			throw new NullPointerException();
		}

		if (size == 1 && (x == -1 || x == 0 || x == 1)) {
			head = null;
			tail = null;
			size = 0;
			return;
		} else if (size == 1) {
			throw new IndexOutOfBoundsException();
		}

		if (x == -1 || x == size) {
			tail = tail.prev;
			tail.next = null;
		} else if (x == 0 || x == 1) {
			head = head.next;
			head.prev = null;
		} else if (x > 1 && x < size) {
			int counter = 0;
			Node temp = head; // temporary node

			// After this loop, temp points at the element which is about to be removed.
			while (temp.next != null && counter < x - 1) {
				temp = temp.next;
				counter++;
			}

			temp.prev.next = temp.next;
			temp.next.prev = temp.prev;
			temp = null;
		} else {
			throw new IndexOutOfBoundsException();
		}

		size--;
	}

	/**
	 * This private inner class provides an iterator for the list of SList.
	 * 
	 * @author Stefan Klbl
	 *
	 */
	private class ListIter implements Iterator<A> {
		private Node current_elem = head; // current position of iterator
		private int counter = 0; // counts the position of last

		/**
		 * This method returns the next element in the iteration.
		 * 
		 * @return next element in the iteration.
		 */
		public A next() {
			if (current_elem == null) {
				return null;
			}

			Node last = current_elem;
			current_elem = current_elem.next;
			counter++;
			return last.elem;
		}

		/**
		 * This method checks if the iteration has more elements.
		 * 
		 * @return true if the iteration has more elements.
		 */
		public boolean hasNext() {
			return current_elem != null;
		}

		/**
		 * This method removes from the underlying collection the last element returned by the iterator.
		 */
		public void remove() {
			SList.this.remove(counter);
			counter--;
		}
	}

	/**
	 * This method creates an iterator for the list.
	 * 
	 * @return an iterator.
	 */
	public Iterator<A> iterator() {
		return new ListIter();
	}
}
